@@ -18,10 +18,10 @@ Developed by Stackmate India in 2021.
 //! ## Building a transaction
 //! 1. Build a transaction with a default fixed fee of 1000 sats
 //! 2. Get weight of the transaction for a given descriptor
-//! 3. Use get absolute fee to get the fee needed to be paid for the transaction given variable fee rate and fixed weight. 
+//! 3. Use get absolute fee to get the fee needed to be paid for the transaction given variable fee rate and fixed weight.
 //! 4. Build transaction with the absolute fee chosen, sign & broadcast.
-//! 
-//! 
+//!
+//!
 //! ### Tor controls are in BETA. Use with caution.
 use std::ffi::{CStr, CString};
 use std::os::raw::c_char;
@@ -47,13 +47,22 @@ use crate::wallet::psbt;
 
 pub mod network;
 use crate::network::fees;
-use crate::network::tor;
+use crate::network::height;
+
+// use crate::network::tor;
 
 /// Generates a mnemonic phrase of a given length. Defaults to 24 words.
 /// A master xprv is created from the mnemonic and passphrase.
-/// - *OUTPUT CONTAINS PRIVATE KEY DATA*
+/// - *OUTPUT*
+/// ```
+/// MasterKey {
+///   fingerprint: String,
+///   mnemonic: String,
+///   xprv: String,
+/// }
+/// ```
 /// # Safety
-/// - This function is unsafe because it dereferences and returns raw pointer.
+/// - This function is unsafe because it dereferences and a returns raw pointer.
 /// - ENSURE that output is passed into cstring_free(ptr: *mut c_char) after use.
 #[no_mangle]
 pub unsafe extern "C" fn generate_master(
@@ -100,9 +109,16 @@ pub unsafe extern "C" fn generate_master(
 }
 
 /// Creates a master xprv given a mnemonic and passphrase.
-/// - *OUTPUT CONTAINS PRIVATE KEY DATA*
+/// - *OUTPUT*
+/// ```
+/// MasterKey {
+///   fingerprint: String,
+///   mnemonic: String,
+///   xprv: String,
+/// }
+/// ```
 /// # Safety
-/// - This function is unsafe because it dereferences and returns raw pointer.
+/// - This function is unsafe because it dereferences and a returns raw pointer.
 /// - ENSURE that result is passed into cstring_free(ptr: *mut c_char) after use.
 #[no_mangle]
 pub unsafe extern "C" fn import_master(
@@ -142,9 +158,17 @@ pub unsafe extern "C" fn import_master(
 /// Derives hardened child keys from a master xprv.
 /// Follows the BIP32 standard of m/purpose'/network'/account'.
 /// Network path is inferred from the master xprv.
-/// - *OUTPUT CONTAINS PRIVATE KEY DATA*
+/// - *OUTPUT*
+/// ```
+/// ChildKeys {
+///   fingerprint: String,
+///   hardened_path: String,
+///   xprv: String,
+///   xpub: String,
+/// }
+/// ```
 /// # Safety
-/// - This function is unsafe because it dereferences and returns raw pointer.
+/// - This function is unsafe because it dereferences and a returns raw pointer.
 /// - ENSURE that result is passed into cstring_free(ptr: *mut c_char) after use.
 #[no_mangle]
 pub unsafe extern "C" fn derive_hardened(
@@ -191,9 +215,15 @@ pub unsafe extern "C" fn derive_hardened(
 /// Compiles a policy into a descriptor of the specified script type.
 /// Use wpkh for a single signature segwit native wallet (default).
 /// Use wsh for a scripted segwit native wallet.
-/// - *OUTPUT CONTAINS PRIVATE KEY DATA*
+/// - *OUTPUT*
+/// ```
+/// WalletPolicy {
+///   policy: String,
+///   descriptor: String,
+/// }
+/// ```
 /// # Safety
-/// - This function is unsafe because it dereferences and returns raw pointer.
+/// - This function is unsafe because it dereferences and a returns raw pointer.
 /// - ENSURE that result is passed into cstring_free(ptr: *mut c_char) after use.
 #[no_mangle]
 pub unsafe extern "C" fn compile(policy: *const c_char, script_type: *const c_char) -> *mut c_char {
@@ -222,8 +252,14 @@ pub unsafe extern "C" fn compile(policy: *const c_char, script_type: *const c_ch
 }
 
 /// Syncs to a remote node and fetches balance of a descriptor wallet.
+/// - *OUTPUT*
+/// ```
+/// WalletBalance {
+///   balance: u64
+/// }
+/// ```
 /// # Safety
-/// - This function is unsafe because it dereferences and returns raw pointer.
+/// - This function is unsafe because it dereferences and a returns raw pointer.
 /// - ENSURE that result is passed into cstring_free(ptr: *mut c_char) after use.
 #[no_mangle]
 pub unsafe extern "C" fn sync_balance(
@@ -259,8 +295,22 @@ pub unsafe extern "C" fn sync_balance(
 }
 
 /// Syncs to a remote node and fetches history of a descriptor wallet.
+/// - *OUTPUT*
+/// ```
+///  WalletHistory{
+///    history: Vec<Transaction {
+///      timestamp: u64,
+///      height: u32,
+///      verified: bool,
+///      txid: String,
+///     received: u64,
+///      sent: u64,
+///      fee: u64,
+///    }>;
+///  }
+/// ```
 /// # Safety
-/// - This function is unsafe because it dereferences and returns raw pointer.
+/// - This function is unsafe because it dereferences and a returns raw pointer.
 /// - ENSURE that result is passed into cstring_free(ptr: *mut c_char) after use.
 #[no_mangle]
 pub unsafe extern "C" fn sync_history(
@@ -297,8 +347,14 @@ pub unsafe extern "C" fn sync_history(
 
 /// Gets a new address for a descriptor wallet at a given index.
 /// Client must keep track of address indexes and ENSURE prevention of address reuse.
+/// - *OUTPUT*
+/// ```
+/// WalletAddress {
+///   address: String,
+/// }
+/// ```
 /// # Safety
-/// - This function is unsafe because it dereferences and returns raw pointer.
+/// - This function is unsafe because it dereferences and a returns raw pointer.
 /// - ENSURE that result is passed into cstring_free(ptr: *mut c_char) after use.
 #[no_mangle]
 pub unsafe extern "C" fn get_address(
@@ -349,8 +405,15 @@ pub unsafe extern "C" fn get_address(
 }
 
 /// Gets the current network fee (in sats/vbyte) for a given confirmation target.
+/// - *OUTPUT*
+/// ```  
+///  NetworkFee {
+///    rate: f32,
+///    absolute: Option<u64>,
+///  }
+/// ```
 /// # Safety
-/// - This function is unsafe because it dereferences and returns raw pointer.
+/// - This function is unsafe because it dereferences and a returns raw pointer.
 /// - ENSURE that result is passed into cstring_free(ptr: *mut c_char) after use.
 #[no_mangle]
 pub unsafe extern "C" fn estimate_network_fee(
@@ -402,8 +465,15 @@ pub unsafe extern "C" fn estimate_network_fee(
 }
 
 /// Converts a given fee_rate (in sats/vbyte) to absolute fee (in sats); given some transaction weight.
+/// - *OUTPUT*
+/// ```  
+///  NetworkFee {
+///    rate: f32,
+///    absolute: Option<u64>,
+///  }
+/// ```
 /// # Safety
-/// - This function is unsafe because it dereferences and returns raw pointer.
+/// - This function is unsafe because it dereferences and a returns raw pointer.
 /// - ENSURE that result is passed into cstring_free(ptr: *mut c_char) after use.
 #[no_mangle]
 pub unsafe extern "C" fn fee_rate_to_absolute(
@@ -426,8 +496,15 @@ pub unsafe extern "C" fn fee_rate_to_absolute(
 }
 
 /// Converts a given absolute_fee (in sats) to fee rate (in sats/vbyte); given some transaction weight.
+/// - *OUTPUT*
+/// ```  
+///  NetworkFee {
+///    rate: f32,
+///    absolute: Option<u64>,
+///  }
+/// ```
 /// # Safety
-/// - This function is unsafe because it dereferences and returns raw pointer.
+/// - This function is unsafe because it dereferences and a returns raw pointer.
 /// - ENSURE that result is passed into cstring_free(ptr: *mut c_char) after use.
 #[no_mangle]
 pub unsafe extern "C" fn fee_absolute_to_rate(
@@ -450,8 +527,14 @@ pub unsafe extern "C" fn fee_absolute_to_rate(
 }
 
 /// Gets the weight of a transaction built with a given deposit-descriptor.
+/// - *OUTPUT*
+/// ```  
+///  TransactionWeight {
+///     weight: usize,
+///  }
+/// ```
 /// # Safety
-/// - This function is unsafe because it dereferences and returns raw pointer.
+/// - This function is unsafe because it dereferences and a returns raw pointer.
 /// - ENSURE that result is passed into cstring_free(ptr: *mut c_char) after use.
 #[no_mangle]
 pub unsafe extern "C" fn get_weight(
@@ -479,8 +562,15 @@ pub unsafe extern "C" fn get_weight(
 /// Builds a transaction for a given descriptor wallet.
 /// If sweep is set to true, amount value is ignored and will default to None.
 /// Set amount to 0 for sweep.
+/// - *OUTPUT*
+/// ```
+///  WalletPSBT {
+///    pub psbt: String,
+///    pub is_finalized: bool,
+///  }
+/// ```
 /// # Safety
-/// - This function is unsafe because it dereferences and returns raw pointer.
+/// - This function is unsafe because it dereferences and a returns raw pointer.
 /// - ENSURE that result is passed into cstring_free(ptr: *mut c_char) after use.
 #[no_mangle]
 pub unsafe extern "C" fn build_tx(
@@ -558,8 +648,17 @@ pub unsafe extern "C" fn build_tx(
 
 /// Decodes a PSBT and returns all outputs of the transaction and total size.
 /// "miner" is used in the 'to' field of an output to indicate fee.
+/// - *OUTPUT*
+/// ```
+///   DecodedTx{
+///     outputs: Vec<DecodedTxIO {
+///       value: u64,
+///       to: String,
+///     }>
+///   }
+/// ```
 /// # Safety
-/// - This function is unsafe because it dereferences and returns raw pointer.
+/// - This function is unsafe because it dereferences and a returns raw pointer.
 /// - ENSURE that result is passed into cstring_free(ptr: *mut c_char) after use.
 #[no_mangle]
 pub unsafe extern "C" fn decode_psbt(network: *const c_char, psbt: *const c_char) -> *mut c_char {
@@ -588,8 +687,15 @@ pub unsafe extern "C" fn decode_psbt(network: *const c_char, psbt: *const c_char
 
 /// Signs a PSBT with a descriptor.
 /// Can only be used with descriptors containing private key(s).
+/// - *OUTPUT*
+/// ```
+///  WalletPSBT {
+///    pub psbt: String,
+///    pub is_finalized: bool,
+///  }
+/// ```
 /// # Safety
-/// - This function is unsafe because it dereferences and returns raw pointer.
+/// - This function is unsafe because it dereferences and a returns raw pointer.
 /// - ENSURE that result is passed into cstring_free(ptr: *mut c_char) after use.
 #[no_mangle]
 pub unsafe extern "C" fn sign_tx(
@@ -633,8 +739,14 @@ pub unsafe extern "C" fn sign_tx(
 }
 
 /// Broadcasts a signed transaction to a remote node.
+/// - *OUTPUT*
+/// ```
+///  Txid {
+///    pub txid: String,
+///  }
+/// ```
 /// # Safety
-/// - This function is unsafe because it dereferences and returns raw pointer.
+/// - This function is unsafe because it dereferences and a returns raw pointer.
 /// - ENSURE that result is passed into cstring_free(ptr: *mut c_char) after use.
 #[no_mangle]
 pub unsafe extern "C" fn broadcast_tx(
@@ -679,8 +791,12 @@ pub unsafe extern "C" fn broadcast_tx(
 
 /// Checks if an extended public key is valid.
 /// Do not use the key source while checking an xpub i.e. remove [fingerprint/derivation/path/values] and only provide the xpub/tpub.
+/// - *OUTPUT*
+/// ```
+/// "true" | "false"
+/// ```
 /// # Safety
-/// - This function is unsafe because it dereferences and returns raw pointer.
+/// - This function is unsafe because it dereferences and a returns raw pointer.
 /// - ENSURE that result is passed into cstring_free(ptr: *mut c_char) after use.
 #[no_mangle]
 pub unsafe extern "C" fn check_xpub(xpub: *const c_char) -> *mut c_char {
@@ -696,62 +812,138 @@ pub unsafe extern "C" fn check_xpub(xpub: *const c_char) -> *mut c_char {
   }
 }
 
-
-/// Switch on tor daemon. 
-/// BETA: Careful with this.
+/// Gets the current block height.
+/// - *OUTPUT*
+/// ```  
+///  BlockHeight {
+///    height: u32,
+///  }
+/// ```
 /// # Safety
-/// - This function is unsafe because it dereferences and returns raw pointer.
+/// - This function is unsafe because it dereferences and a returns raw pointer.
 /// - ENSURE that result is passed into cstring_free(ptr: *mut c_char) after use.
 #[no_mangle]
-pub unsafe extern "C" fn tor_start(tmp_path: *mut c_char) -> *mut c_char {
-  let tmp_path_cstr = CStr::from_ptr(tmp_path);
-  let tmp_path: &str = match tmp_path_cstr.to_str() {
+pub unsafe extern "C" fn get_height(
+  network: *const c_char,
+  node_address: *const c_char,
+) -> *mut c_char {
+  let network_cstr = CStr::from_ptr(network);
+  let network: &str = match network_cstr.to_str() {
     Ok(string) => string,
-    Err(_) => "/tmp",
+    Err(_) => "test",
+  };
+  let network_enum = match network {
+    "main" => Network::Bitcoin,
+    _ => Network::Testnet,
+  };
+  let node_address_cstr = CStr::from_ptr(node_address);
+  let node_address: &str = match node_address_cstr.to_str() {
+    Ok(string) => {
+      if string == DEFAULT {
+        match network_enum {
+          Network::Bitcoin => DEFAULT_MAINNET_NODE,
+          _ => DEFAULT_TESTNET_NODE,
+        }
+      } else {
+        string
+      }
+    }
+    Err(_) => match network_enum {
+      Network::Bitcoin => DEFAULT_MAINNET_NODE,
+      _ => DEFAULT_TESTNET_NODE,
+    },
   };
 
-  let control_key = tor::start(tmp_path);
-  CString::new(control_key).unwrap().into_raw()
+  let config = match WalletConfig::new("/0/*", node_address, None) {
+    Ok(conf) => conf,
+    Err(e) => return S5Error::new(ErrorKind::Internal, &e.message).c_stringify(),
+  };
+  match height::get_height(config) {
+    Ok(result) => result.c_stringify(),
+    Err(e) => e.c_stringify(),
+  }
 }
 
+/// Convert days to bitcoin blocks as height. *Does not consider the current block height*. Add to get_height to get the expected block height after the given time in days.
+/// - *OUTPUT*
+/// ```  
+///  BlockHeight {
+///    height: u32,
+///  }
+/// ```
+/// # Safety
+/// - This function is unsafe because it dereferences and a returns raw pointer.
+/// - ENSURE that result is passed into cstring_free(ptr: *mut c_char) after use.
+#[no_mangle]
+pub unsafe extern "C" fn days_to_blocks(days: *const c_char) -> *mut c_char {
+  let days_cstr = CStr::from_ptr(days);
+  let days: u32 = match days_cstr.to_str() {
+    Ok(string) => match string.parse::<u32>() {
+      Ok(i) => i,
+      Err(_) => {
+        return S5Error::new(ErrorKind::Input, "Days must be a stringified uint32").c_stringify()
+      }
+    },
+    Err(_) => return S5Error::new(ErrorKind::Input, "Days").c_stringify(),
+  };
+
+  height::BlockHeight { height: days * 144 }.c_stringify()
+}
+
+/// Switch on tor daemon.
+/// BETA: Careful with this.
+/// # Safety
+/// - This function is unsafe because it dereferences and a returns raw pointer.
+/// - ENSURE that result is passed into cstring_free(ptr: *mut c_char) after use.
+// #[no_mangle]
+// pub unsafe extern "C" fn tor_start(tmp_path: *mut c_char) -> *mut c_char {
+//   let tmp_path_cstr = CStr::from_ptr(tmp_path);
+//   let tmp_path: &str = match tmp_path_cstr.to_str() {
+//     Ok(string) => string,
+//     Err(_) => "/tmp",
+//   };
+
+//   let control_key = tor::start(tmp_path);
+//   CString::new(control_key).unwrap().into_raw()
+// }
+
 /// Get bootstrap progress from tor daemon. Wait ~1s after calling tor_start() before calling this.
 /// BETA: Careful with this.
 /// # Safety
-/// - This function is unsafe because it dereferences and returns raw pointer.
+/// - This function is unsafe because it dereferences and a returns raw pointer.
 /// - ENSURE that result is passed into cstring_free(ptr: *mut c_char) after use.
-#[no_mangle]
-pub unsafe extern "C" fn tor_progress(control_key: *mut c_char) -> *mut c_char {
-  let control_key_cstr = CStr::from_ptr(control_key);
-  let control_key: &str = match control_key_cstr.to_str() {
-    Ok(string) => string,
-    Err(_) => return S5Error::new(ErrorKind::Input, "Control-Key").c_stringify(),
-  };
-  match tor::bootstrap_progress(control_key){
-    Ok(result) => CString::new(result.to_string()).unwrap().into_raw(),
-    Err(e) => e.c_stringify()
-  }
-}
+// #[no_mangle]
+// pub unsafe extern "C" fn tor_progress(control_key: *mut c_char) -> *mut c_char {
+//   let control_key_cstr = CStr::from_ptr(control_key);
+//   let control_key: &str = match control_key_cstr.to_str() {
+//     Ok(string) => string,
+//     Err(_) => return S5Error::new(ErrorKind::Input, "Control-Key").c_stringify(),
+//   };
+//   match tor::bootstrap_progress(control_key){
+//     Ok(result) => CString::new(result.to_string()).unwrap().into_raw(),
+//     Err(e) => e.c_stringify()
+//   }
+// }
 
 /// Shutdown tor daemon.
 /// BETA: Careful with this.
 /// # Safety
-/// - This function is unsafe because it dereferences and returns raw pointer.
+/// - This function is unsafe because it dereferences and a returns raw pointer.
 /// - ENSURE that result is passed into cstring_free(ptr: *mut c_char) after use.
-#[no_mangle]
-pub unsafe extern "C" fn tor_stop(control_key: *mut c_char) -> *mut c_char {
-  let control_key_cstr = CStr::from_ptr(control_key);
-  let control_key: &str = match control_key_cstr.to_str() {
-    Ok(string) => string,
-    Err(_) => return S5Error::new(ErrorKind::Input, "Control-Key").c_stringify(),
-  };
+// #[no_mangle]
+// pub unsafe extern "C" fn tor_stop(control_key: *mut c_char) -> *mut c_char {
+//   let control_key_cstr = CStr::from_ptr(control_key);
+//   let control_key: &str = match control_key_cstr.to_str() {
+//     Ok(string) => string,
+//     Err(_) => return S5Error::new(ErrorKind::Input, "Control-Key").c_stringify(),
+//   };
 
-  match tor::shutdown(control_key){
-    Ok(result) => CString::new(result.to_string()).unwrap().into_raw(),
-    Err(e) => e.c_stringify()
-  }
-}
+//   match tor::shutdown(control_key){
+//     Ok(result) => CString::new(result.to_string()).unwrap().into_raw(),
+//     Err(e) => e.c_stringify()
+//   }
+// }
 
- 
 /// After using any other function, pass the output pointer into cstring_free(ptr: *mut c_char) to clear memory.
 /// ALWAYS use this in combination with any other function.
 /// Failure to do so can lead to memory bugs.
@@ -888,4 +1080,20 @@ mod tests {
       // assert_eq!(history.history.len(),3);
     }
   }
+
+  #[test]
+  fn test_days_to_blocks() {
+    unsafe {
+      let one_day = "1";
+      let blocks_ptr = days_to_blocks(CString::new(one_day).unwrap().into_raw());
+      let blocks_str = CStr::from_ptr(blocks_ptr).to_str().unwrap();
+      let blocks: height::BlockHeight = serde_json::from_str(blocks_str).unwrap();
+      assert_eq!(blocks.height, 144);
+      let one_year_as_days = "365";
+      let blocks_ptr = days_to_blocks(CString::new(one_year_as_days).unwrap().into_raw());
+      let blocks_str = CStr::from_ptr(blocks_ptr).to_str().unwrap();
+      let blocks: height::BlockHeight = serde_json::from_str(blocks_str).unwrap();
+      assert_eq!(blocks.height, 52560);
+    }
+  }
 }
